@@ -12,8 +12,8 @@
 
 
 char bcm5700_driver[] = "bcm5700";
-char bcm5700_version[] = "5.0.16";
-char bcm5700_date[] = "(01/03/03)";
+char bcm5700_version[] = "6.0.2";
+char bcm5700_date[] = "(02/14/03)";
 
 #define B57UM
 #include "mm.h"
@@ -230,7 +230,7 @@ bcm5700_lock(PUM_DEVICE_BLOCK pUmDevice)
 }
 
 inline void
-bcm5700_unlock(PUM_DEVICE_BLOCK pUmDevice, long flags)
+bcm5700_unlock(PUM_DEVICE_BLOCK pUmDevice, unsigned long flags)
 {
 	if (pUmDevice->do_global_lock) {
 		spin_unlock_irqrestore(&pUmDevice->global_lock, flags);
@@ -393,11 +393,12 @@ STATIC int bcm5700_ioctl(struct net_device *dev, struct ifreq *rq, int cmd);
 STATIC void bcm5700_set_rx_mode(struct net_device *dev);
 STATIC int bcm5700_set_mac_addr(struct net_device *dev, void *p);
 STATIC int bcm5700_change_mtu(struct net_device *dev, int new_mtu);
-STATIC int replenish_rx_buffers(PUM_DEVICE_BLOCK pUmDevice);
+STATIC int replenish_rx_buffers(PUM_DEVICE_BLOCK pUmDevice, int max);
 STATIC int bcm5700_freemem(struct net_device *dev);
 #ifdef BCM_INT_COAL
 STATIC int bcm5700_adapt_coalesce(PUM_DEVICE_BLOCK pUmDevice);
 #endif
+STATIC int bcm5700_init_counters(PUM_DEVICE_BLOCK pUmDevice);
 #ifdef BCM_VLAN
 STATIC void bcm5700_vlan_rx_register(struct net_device *dev, struct vlan_group *vlgrp);
 STATIC void bcm5700_vlan_rx_kill_vid(struct net_device *dev, uint16_t vid);
@@ -445,6 +446,7 @@ typedef enum {
 	TC999T,
 	NC6770,
 	NC7760,
+	NC7761,
 	NC7770,
 	NC7771,
 	NC7780,
@@ -496,12 +498,13 @@ static struct {
 	{ "3Com 3C998-T Dual Port 10/100/1000 PCI-X Server NIC" },
 	{ "3Com 3C998-SX Dual Port 1000-SX PCI-X Server NIC" },
 	{ "3Com 3C999-T Quad Port 10/100/1000 PCI-X Server NIC" },
-	{ "Compaq NC6770 Gigabit Server Adapter" },
-	{ "Compaq NC7760 Gigabit Server Adapter" },
-	{ "Compaq NC7770 Gigabit Server Adapter" },
-	{ "Compaq NC7771 Gigabit Server Adapter" },
-	{ "Compaq NC7780 Gigabit Server Adapter" },
-	{ "Compaq NC7781 Gigabit Server Adapter" },
+	{ "HP NC6770 Gigabit Server Adapter" },
+	{ "HP NC7760 Gigabit Server Adapter" },
+	{ "HP NC7761 Gigabit Server Adapter" },
+	{ "HP NC7770 Gigabit Server Adapter" },
+	{ "HP NC7771 Gigabit Server Adapter" },
+	{ "HP NC7780 Gigabit Server Adapter" },
+	{ "HP NC7781 Gigabit Server Adapter" },
 	{ "HP NC7772 Gigabit Server Adapter" },
 	{ "HP NC7782 Gigabit Server Adapter" },
 	{ "HP NC7783 Gigabit Server Adapter" },
@@ -510,7 +513,7 @@ static struct {
 	{ "Broadcom BCM5704 1000Base-SX" },
 	{ "Broadcom BCM5705 1000Base-T" },
 	{ "Broadcom BCM5705M 1000Base-T" },
-	{ "Broadcom BCM5782 1000Base-T" },
+	{ "Broadcom NetXtreme Gigabit Ethernet for hp" },
 	{ 0 },
 	};
 
@@ -578,8 +581,12 @@ static struct pci_device_id bcm5700_pci_tbl[] __devinitdata = {
 	{0x14e4, 0x16a8, 0x14e4, 0x16a8, 0, 0, BCM5704S },
 	{0x14e4, 0x16a8, 0x10b7, 0x2001, 0, 0, TC998SX },
 	{0x14e4, 0x16a8, PCI_ANY_ID, PCI_ANY_ID, 0, 0, BCM5704S },
+	{0x14e4, 0x1653, 0x0e11, 0x00e3, 0, 0, NC7761 },
 	{0x14e4, 0x1653, PCI_ANY_ID, PCI_ANY_ID, 0, 0, BCM5705 },
+	{0x14e4, 0x1654, 0x0e11, 0x00e3, 0, 0, NC7761 },
+	{0x14e4, 0x1654, PCI_ANY_ID, PCI_ANY_ID, 0, 0, BCM5705 },
 	{0x14e4, 0x165d, PCI_ANY_ID, PCI_ANY_ID, 0, 0, BCM5705M },
+	{0x14e4, 0x165e, PCI_ANY_ID, PCI_ANY_ID, 0, 0, BCM5705M },
 	{0x14e4, 0x1696, PCI_ANY_ID, PCI_ANY_ID, 0, 0, BCM5782 },
 	{0,}
 };
@@ -650,9 +657,11 @@ static int __devinit bcm5700_init_board(struct pci_dev *pdev,
 	dev->mtu = mtu[board_idx];
 
 	if (pci_find_device(0x8086, 0x2418, NULL) ||
-		pci_find_device(0x8086, 0x2428, NULL)) {
+		pci_find_device(0x8086, 0x2428, NULL) ||
+		pci_find_device(0x8086, 0x244e, NULL) ||
+		pci_find_device(0x8086, 0x2448, NULL)) {
 
-		/* Found ICH or ICH0 */
+		/* Found ICH, ICH0, or ICH2 */
 		pDevice->UndiFix = 1;
 	}
 
@@ -957,19 +966,14 @@ bcm5700_open(struct net_device *dev)
 	}
 
 	pUmDevice->opened = 1;
-#if TIGON3_DEBUG
-	pUmDevice->tx_zc_count = 0;
-	pUmDevice->tx_chksum_count = 0;
-	pUmDevice->tx_himem_count = 0;
-	pUmDevice->rx_good_chksum_count = 0;
-	pUmDevice->rx_bad_chksum_count = 0;
-#endif
 	if (LM_InitializeAdapter(pDevice) != LM_STATUS_SUCCESS) {
 		free_irq(dev->irq, dev);
 		bcm5700_freemem(dev);
 		return -EAGAIN;
 	}
 
+	bcm5700_init_counters(pUmDevice);
+
 	if (pDevice->UndiFix) {
 		printk(KERN_INFO "%s: Using indirect register access\n", dev->name);
 	}
@@ -1133,6 +1137,32 @@ bcm5700_timer(unsigned long data)
 	add_timer(&pUmDevice->timer);
 }
 
+STATIC int
+bcm5700_init_counters(PUM_DEVICE_BLOCK pUmDevice)
+{
+	PLM_DEVICE_BLOCK pDevice = &pUmDevice->lm_dev;
+
+#ifdef BCM_INT_COAL
+	pUmDevice->rx_curr_coalesce_frames = pDevice->RxMaxCoalescedFrames;
+	pUmDevice->rx_curr_coalesce_ticks = pDevice->RxCoalescingTicks;
+	pUmDevice->tx_curr_coalesce_frames = pDevice->TxMaxCoalescedFrames;
+	pUmDevice->rx_last_cnt = 0;
+	pUmDevice->tx_last_cnt = 0;
+#endif
+	pDevice->PhyCrcCount = 0;
+#if TIGON3_DEBUG
+	pUmDevice->tx_zc_count = 0;
+	pUmDevice->tx_chksum_count = 0;
+	pUmDevice->tx_himem_count = 0;
+	pUmDevice->rx_good_chksum_count = 0;
+	pUmDevice->rx_bad_chksum_count = 0;
+#endif
+#ifdef BCM_TSO
+	pUmDevice->tso_pkt_count = 0;
+#endif
+	return 0;
+}
+
 #ifdef BCM_INT_COAL
 STATIC int
 bcm5700_adapt_coalesce(PUM_DEVICE_BLOCK pUmDevice)
@@ -1145,7 +1175,7 @@ bcm5700_adapt_coalesce(PUM_DEVICE_BLOCK pUmDevice)
 	rx_curr_cnt = pDevice->pStatsBlkVirt->ifHCInUcastPkts.Low;
 	tx_curr_cnt = pDevice->pStatsBlkVirt->ifHCOutUcastPkts.Low;
 	if ((rx_curr_cnt <= pUmDevice->rx_last_cnt) ||
-		(tx_curr_cnt <= pUmDevice->tx_last_cnt)) {
+		(tx_curr_cnt < pUmDevice->tx_last_cnt)) {
 
 		/* skip if there is counter rollover */
 		pUmDevice->rx_last_cnt = rx_curr_cnt;
@@ -1160,9 +1190,6 @@ bcm5700_adapt_coalesce(PUM_DEVICE_BLOCK pUmDevice)
 	pUmDevice->rx_last_cnt = rx_curr_cnt;
 	pUmDevice->tx_last_cnt = tx_curr_cnt;
 
-	if (total_delta < rx_delta)
-		return 0;
-
 	if (total_delta < ADAPTIVE_LO_PKT_THRESH) {
 		if (pUmDevice->rx_curr_coalesce_frames !=
 			ADAPTIVE_LO_RX_MAX_COALESCED_FRAMES) {
@@ -1253,6 +1280,7 @@ bcm5700_reset(struct net_device *dev)
 	bcm5700_intr_off(pUmDevice);
 	flags = bcm5700_phy_lock(pUmDevice);
 	LM_ResetAdapter(pDevice);	
+	bcm5700_init_counters(pUmDevice);
 	if (memcmp(dev->dev_addr, pDevice->NodeAddress, 6)) {
 		LM_SetMacAddress(pDevice, dev->dev_addr);
 	}
@@ -1309,7 +1337,7 @@ bcm5700_start_xmit(struct sk_buff *skb, struct net_device *dev)
 #endif
 #ifdef BCM_TSO
 	LM_UINT32 mss;
-	uint16_t ip_tcp_len, tcp_opt_len, last_seg_len, tcp_seg_flags;
+	uint16_t ip_tcp_len, tcp_opt_len, tcp_seg_flags;
 #endif
 
 	if ((pDevice->LinkStatus == LM_STATUS_LINK_DOWN) ||
@@ -1423,8 +1451,6 @@ bcm5700_start_xmit(struct sk_buff *skb, struct net_device *dev)
 			}
 		}
 		else {
-			last_seg_len = (skb->len - ip_tcp_len - tcp_opt_len -
-				14) % mss;
 			mss += tcp_opt_len;
 		}
 		pPacket->u.Tx.MaxSegmentSize = mss | tcp_seg_flags;
@@ -1540,7 +1566,8 @@ bcm5700_interrupt(int irq, void *dev_instance, struct pt_regs *regs)
 		pDevice->QueueAgain) &&
 		(!test_and_set_bit(0, &pUmDevice->tasklet_busy))) {
 
-		replenish_rx_buffers(pUmDevice);
+		replenish_rx_buffers(pUmDevice,
+			pUmDevice->rx_buf_repl_isr_limit);
 		clear_bit(0, (void*)&pUmDevice->tasklet_busy);
 	}
 	else if ((repl_buf_count > pUmDevice->rx_buf_repl_thresh) &&
@@ -1551,7 +1578,7 @@ bcm5700_interrupt(int irq, void *dev_instance, struct pt_regs *regs)
 	}
 #else
 	if (QQ_GetEntryCnt(&pUmDevice->rx_out_of_buf_q.Container)) {
-		replenish_rx_buffers(pUmDevice);
+		replenish_rx_buffers(pUmDevice, 0);
 	}
 
 	if (QQ_GetEntryCnt(&pDevice->RxPacketFreeQ.Container) ||
@@ -1584,7 +1611,7 @@ bcm5700_tasklet(unsigned long data)
 	}
 
 	pUmDevice->tasklet_pending = 0;
-	replenish_rx_buffers(pUmDevice);
+	replenish_rx_buffers(pUmDevice, 0);
 	clear_bit(0, &pUmDevice->tasklet_busy);
 }
 #endif
@@ -2131,6 +2158,9 @@ STATIC int bcm5700_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 	case SIOCGMIIREG:
 #endif
 	case SIOCDEVPRIVATE+1:		/* Read the specified MII register. */
+		if (pDevice->EnableTbi)
+			return -EOPNOTSUPP;
+
 		flags = bcm5700_phy_lock(pUmDevice);
 		LM_ReadPhy(pDevice, data[1] & 0x1f, (LM_UINT32 *) &value);
 		bcm5700_phy_unlock(pUmDevice, flags);
@@ -2143,6 +2173,10 @@ STATIC int bcm5700_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
 	case SIOCDEVPRIVATE+2:		/* Write the specified MII register */
 		if (!capable(CAP_NET_ADMIN))
 			return -EPERM;
+
+		if (pDevice->EnableTbi)
+			return -EOPNOTSUPP;
+
 		flags = bcm5700_phy_lock(pUmDevice);
 		LM_WritePhy(pDevice, data[1] & 0x1f, data[2]);
 		bcm5700_phy_unlock(pUmDevice, flags);
@@ -2420,6 +2454,7 @@ STATIC int bcm5700_change_mtu(struct net_device *dev, int new_mtu)
 
 	if (reinit) {
 		LM_InitializeAdapter(pDevice);
+		bcm5700_init_counters(pUmDevice);
 		if (memcmp(dev->dev_addr, pDevice->NodeAddress, 6)) {
 			LM_SetMacAddress(pDevice, dev->dev_addr);
 		}
@@ -2734,11 +2769,16 @@ MM_InitializeUmPackets(PLM_DEVICE_BLOCK pDevice)
 		/* reallocate buffers in the ISR */
 		pUmDevice->rx_buf_repl_thresh = 0;
 		pUmDevice->rx_buf_repl_panic_thresh = 0;
+		pUmDevice->rx_buf_repl_isr_limit = 0;
 	}
 	else {
 		pUmDevice->rx_buf_repl_thresh = pDevice->RxPacketDescCnt / 8;
 		pUmDevice->rx_buf_repl_panic_thresh =
-			pDevice->RxPacketDescCnt / 2;
+			pDevice->RxPacketDescCnt  * 7 / 8;
+
+		/* This limits the time spent in the ISR when the receiver */
+		/* is in a steady state of being overrun. */
+		pUmDevice->rx_buf_repl_isr_limit = pDevice->RxPacketDescCnt / 8;
 
 		if (pDevice->RxJumboDescCnt != 0) {
 			if (pUmDevice->rx_buf_repl_thresh >=
@@ -3173,13 +3213,14 @@ MM_CoalesceTxBuffer(PLM_DEVICE_BLOCK pDevice, PLM_PACKET pPacket)
 
 /* Returns 1 if not all buffers are allocated */
 STATIC int
-replenish_rx_buffers(PUM_DEVICE_BLOCK pUmDevice)
+replenish_rx_buffers(PUM_DEVICE_BLOCK pUmDevice, int max)
 {
 	PLM_PACKET pPacket;
 	PUM_PACKET pUmPacket;
 	PLM_DEVICE_BLOCK pDevice = (PLM_DEVICE_BLOCK) pUmDevice;
 	struct sk_buff *skb;
 	int queue_rx = 0;
+	int alloc_cnt = 0;
 	int ret = 0;
 
 	while ((pUmPacket = (PUM_PACKET)
@@ -3202,6 +3243,11 @@ replenish_rx_buffers(PUM_DEVICE_BLOCK pUmDevice)
 		skb_reserve(skb, pUmDevice->rx_buf_align);
 		QQ_PushTail(&pDevice->RxPacketFreeQ.Container, pPacket);
 		queue_rx = 1;
+		if (max > 0) {
+			alloc_cnt++;
+			if (alloc_cnt >= max)
+				break;
+		}
 	}
 	if (queue_rx || pDevice->QueueAgain) {
 		LM_QueueRxPackets(pDevice);
