@@ -8,7 +8,6 @@
 /* it under the terms of the GNU General Public License as published by       */
 /* the Free Software Foundation, located in the file LICENSE.                 */
 /*                                                                            */
-/* History:                                                                   */
 /*                                                                            */
 /*  Name: F W _ S T K O F F L D . H                                           */
 /*  Author : Kevin Tran                                                       */
@@ -32,6 +31,8 @@
 /*                              incorrectly computed if the length of data is */
 /*                              less than 6 bytes in the last packetst of     */
 /*                              of a chain of fragmented IP/UDP packets.      */
+/*    12/01/02 Kevin Tran       Fixed a problem where firmware might lockup   */
+/*                              in a certain test scenario with BCM5704.      */
 /******************************************************************************/
 
 #ifndef __FW_STKOFFLD_H__ 
@@ -62,7 +63,7 @@ U32 t3StkOffLdFwText[(0x1ac0/4) + 1] = {
 0xe0005e0, 0x34840002, 0xe000670, 0x0, 
 0x3c030800, 0x90631b78, 0x24020002, 0x3c040800, 
 0x24841acc, 0x14620003, 0x24050001, 0x3c040800, 
-0x24841ac0, 0x2406000a, 0x3821, 0xafa00010, 
+0x24841ac0, 0x24060002, 0x3821, 0xafa00010, 
 0xe000684, 0xafa00014, 0x8f625c50, 0x34420001, 
 0xaf625c50, 0x8f625c90, 0x34420001, 0xaf625c90, 
 0x2402ffff, 0xe000034, 0xaf625404, 0x8fbf0018, 
@@ -168,11 +169,11 @@ U32 t3StkOffLdFwText[(0x1ac0/4) + 1] = {
 0xa4c2000c, 0x3c020800, 0x8c422400, 0x10400003, 
 0x3c024b65, 0xa0001c4, 0x34427654, 0x3c02b49a, 
 0x344289ab, 0xacc2001c, 0x30e2ffff, 0xacc20010, 
-0xe0005ac, 0xc02021, 0x3242ffff, 0x54102b, 
+0xe0005aa, 0xc02021, 0x3242ffff, 0x54102b, 
 0x1440ffa4, 0x0, 0x24020002, 0x3c010800, 
 0xa0001d9, 0xa0221ba8, 0x8ec2083c, 0x24420001, 
 0xa0001d9, 0xaec2083c, 0x14820003, 0x0, 
-0xe0004bb, 0x0, 0x8fbf002c, 0x8fb60028, 
+0xe0004b9, 0x0, 0x8fbf002c, 0x8fb60028, 
 0x8fb50024, 0x8fb40020, 0x8fb3001c, 0x8fb20018, 
 0x8fb10014, 0x8fb00010, 0x3e00008, 0x27bd0030, 
 0x27bdffd0, 0xafbf0028, 0xafb30024, 0xafb20020, 
@@ -253,7 +254,7 @@ U32 t3StkOffLdFwText[(0x1ac0/4) + 1] = {
 0x24420001, 0x3c010800, 0xac221b50, 0x2c620002, 
 0x1040fff7, 0x0, 0x3c024000, 0x2421825, 
 0xaf635c9c, 0x8f625c90, 0x30420002, 0x1440fffc, 
-0x0, 0x12600003, 0x0, 0xe0004bb, 
+0x0, 0x12600003, 0x0, 0xe0004b9, 
 0x0, 0x8fbf0028, 0x8fb30024, 0x8fb20020, 
 0x8fb1001c, 0x8fb00018, 0x3e00008, 0x27bd0030, 
 0x8f634450, 0x3c040800, 0x24841b98, 0x8c820000, 
@@ -284,7 +285,7 @@ U32 t3StkOffLdFwText[(0x1ac0/4) + 1] = {
 0x94421bd4, 0x431021, 0xa5220004, 0x3c020800, 
 0x94421bd0, 0xa5820004, 0x3c020800, 0x8c421bd0, 
 0xa5820006, 0x3c020800, 0x8c422400, 0x3c0d0800, 
-0x8dad23f4, 0x3c0a0800, 0x144000e7, 0x8d4a23f8, 
+0x8dad23f4, 0x3c0a0800, 0x144000e5, 0x8d4a23f8, 
 0x3c020800, 0x94421bd4, 0x4a1821, 0x3063ffff, 
 0x62182b, 0x24020002, 0x10c2000d, 0x1435023, 
 0x3c020800, 0x944223e6, 0x30420009, 0x10400008, 
@@ -308,128 +309,128 @@ U32 t3StkOffLdFwText[(0x1ac0/4) + 1] = {
 0x25670008, 0x109102a, 0x1440fff3, 0x0, 
 0x30820001, 0x10400005, 0x61c02, 0xa0e00001, 
 0x94e20000, 0xc23021, 0x61c02, 0x30c2ffff, 
-0x623021, 0x61402, 0xc23021, 0xa00047b, 
-0x30c6ffff, 0x24020002, 0x14c20083, 0x0, 
+0x623021, 0x61402, 0xc23021, 0xa000479, 
+0x30c6ffff, 0x24020002, 0x14c20081, 0x0, 
 0x3c020800, 0x8c42241c, 0x14400007, 0x0, 
 0x3c020800, 0x944223e2, 0x95230002, 0x1e21023, 
-0x10620079, 0x0, 0x3c020800, 0x944223e2, 
+0x10620077, 0x0, 0x3c020800, 0x944223e2, 
 0x1e21023, 0xa5220002, 0x3c020800, 0x8c42241c, 
-0x1040001d, 0x1203821, 0x8dc70010, 0x3c020800, 
-0x94421ba6, 0x31e4ffff, 0xe04021, 0x72c02, 
-0xaa1821, 0x441023, 0x623823, 0x71c02, 
-0x30e2ffff, 0x623821, 0x71402, 0xe23821, 
-0x71027, 0xa522000a, 0x3102ffff, 0x3c030800, 
-0x946323e4, 0x453023, 0xe02821, 0x832023, 
-0x8d2021, 0xc43021, 0x61c02, 0x30c2ffff, 
-0xa00047b, 0x623021, 0x4021, 0x3082ffff, 
-0x21042, 0x18400008, 0x3021, 0x401821, 
-0x94e20000, 0x25080001, 0xc23021, 0x103102a, 
-0x1440fffb, 0x24e70002, 0x61c02, 0x30c2ffff, 
-0x623021, 0x61402, 0xc23021, 0xc02821, 
-0x61027, 0xa522000a, 0x3021, 0x2527000c, 
-0x4021, 0x94e20000, 0x25080001, 0xc23021, 
-0x2d020004, 0x1440fffb, 0x24e70002, 0x95220002, 
-0x4021, 0x91230009, 0x442023, 0x1803821, 
-0x3082ffff, 0xa4e00010, 0x3c040800, 0x948423e4, 
-0x621821, 0xc33021, 0x61c02, 0x30c2ffff, 
-0x623021, 0x61c02, 0x3c020800, 0x944223e0, 
-0xc34821, 0x441023, 0x21fc2, 0x431021, 
-0x21043, 0x18400010, 0x3021, 0x402021, 
-0x94e20000, 0x24e70002, 0xc23021, 0x30e2007f, 
-0x14400006, 0x25080001, 0x8d630000, 0x3c02007f, 
-0x3442ff80, 0x625824, 0x25670008, 0x104102a, 
-0x1440fff3, 0x0, 0x3c020800, 0x944223fc, 
-0xc23021, 0x3122ffff, 0xc23021, 0x61c02, 
-0x30c2ffff, 0x623021, 0x61402, 0xc23021, 
-0xc04021, 0x61027, 0xa5820010, 0xadc00014, 
-0xa00049b, 0xadc00000, 0x8dc70010, 0xe04021, 
-0x11400007, 0x72c02, 0xaa3021, 0x61402, 
-0x30c3ffff, 0x433021, 0x61402, 0xc22821, 
-0x51027, 0xa522000a, 0x3c030800, 0x946323e4, 
-0x3102ffff, 0x1e21021, 0x433023, 0xcd3021, 
+0x1040001a, 0x31e3ffff, 0x8dc70010, 0x3c020800, 
+0x94421ba6, 0xe04021, 0x72c02, 0xaa2021, 
+0x431023, 0x823823, 0x72402, 0x30e2ffff, 
+0x823821, 0x71027, 0xa522000a, 0x3102ffff, 
+0x3c040800, 0x948423e4, 0x453023, 0xe02821, 
+0x641823, 0x6d1821, 0xc33021, 0x61c02, 
+0x30c2ffff, 0xa000479, 0x623021, 0x1203821, 
+0x4021, 0x3082ffff, 0x21042, 0x18400008, 
+0x3021, 0x401821, 0x94e20000, 0x25080001, 
+0xc23021, 0x103102a, 0x1440fffb, 0x24e70002, 
 0x61c02, 0x30c2ffff, 0x623021, 0x61402, 
-0xc23021, 0xc04021, 0x61027, 0xa5820010, 
-0x3102ffff, 0x51c00, 0x431025, 0xadc20010, 
-0x3c020800, 0x8c422404, 0x10400002, 0x25e2fff2, 
-0xa5c20034, 0x3c020800, 0x8c4223f8, 0x3c040800, 
-0x8c8423f4, 0x24420001, 0x3c010800, 0xac2223f8, 
-0x3c020800, 0x8c421bd0, 0x3303ffff, 0x832021, 
-0x3c010800, 0xac2423f4, 0x431821, 0x62102b, 
-0x10400003, 0x2482ffff, 0x3c010800, 0xac2223f4, 
-0x3c010800, 0xac231bd0, 0x3e00008, 0x27bd0020, 
-0x27bdffb8, 0x3c050800, 0x24a51ba8, 0xafbf0044, 
-0xafbe0040, 0xafb7003c, 0xafb60038, 0xafb50034, 
-0xafb40030, 0xafb3002c, 0xafb20028, 0xafb10024, 
-0xafb00020, 0x90a30000, 0x24020003, 0x146200d5, 
-0x0, 0x3c090800, 0x95291ba6, 0x3c020800, 
-0x944223e0, 0x3c030800, 0x8c631bc0, 0x3c040800, 
-0x8c841bbc, 0x1221023, 0x64182a, 0xa7a9001e, 
-0x106000c8, 0xa7a20016, 0x24be0020, 0x97b6001e, 
-0x24b30018, 0x24b70014, 0x8fc20000, 0x14400008, 
-0x0, 0x8fc2fff8, 0x97a30016, 0x8fc4fff4, 
-0x431021, 0x82202a, 0x148000ba, 0x0, 
-0x97d50818, 0x32a2ffff, 0x104000ad, 0x9021, 
-0x40a021, 0x8821, 0xe00062d, 0x0, 
-0x403021, 0x14c00007, 0x0, 0x3c020800, 
-0x8c4223ec, 0x24420001, 0x3c010800, 0xa0005a0, 
-0xac2223ec, 0x3c100800, 0x2118021, 0x8e101bd8, 
-0x9608000a, 0x31020040, 0x10400004, 0x2407180c, 
-0x8e02000c, 0x2407188c, 0xacc20018, 0x31021000, 
-0x10400004, 0x34e32000, 0x81040, 0x3042c000, 
-0x623825, 0x31020080, 0x54400001, 0x34e70010, 
-0x3c020800, 0x511021, 0x8c421be0, 0x3c030800, 
-0x711821, 0x8c631be4, 0x21500, 0x31c00, 
-0x431025, 0xacc20014, 0x96040008, 0x3242ffff, 
-0x821021, 0x282102a, 0x14400002, 0x2b22823, 
-0x802821, 0x8e020000, 0x2459021, 0xacc20000, 
-0x8e020004, 0xc02021, 0x26310010, 0xac820004, 
-0x30e2ffff, 0xac800008, 0xa485000e, 0xac820010, 
-0x24020305, 0xe0005ac, 0xa482000c, 0x3242ffff, 
-0x54102b, 0x1440ffc0, 0x3242ffff, 0xa000598, 
-0x0, 0x8e620000, 0x8e63fffc, 0x43102a, 
-0x1040006c, 0x0, 0x8e62fff0, 0x28900, 
-0x3c100800, 0x2118021, 0xe00062d, 0x8e101bd8, 
-0x403021, 0x14c00005, 0x0, 0x8e62082c, 
-0x24420001, 0xa0005a0, 0xae62082c, 0x9608000a, 
-0x31020040, 0x10400004, 0x2407180c, 0x8e02000c, 
-0x2407188c, 0xacc20018, 0x31021000, 0x10400004, 
-0x34e32000, 0x81040, 0x3042c000, 0x623825, 
-0x3c020800, 0x511021, 0x8c421be0, 0x3c030800, 
-0x711821, 0x8c631be4, 0x21500, 0x31c00, 
-0x431025, 0xacc20014, 0x8e63fff4, 0x96020008, 
-0x432023, 0x3242ffff, 0x3083ffff, 0x431021, 
-0x2c2102a, 0x10400003, 0x802821, 0x97a9001e, 
-0x1322823, 0x8e620000, 0x30a4ffff, 0x441021, 
-0xae620000, 0xa4c5000e, 0x8e020000, 0xacc20000, 
-0x8e020004, 0x8e63fff4, 0x431021, 0xacc20004, 
-0x8e63fff4, 0x96020008, 0x641821, 0x62102a, 
-0x14400006, 0x2459021, 0x8e62fff0, 0xae60fff4, 
-0x24420001, 0xa00057b, 0xae62fff0, 0xae63fff4, 
-0xacc00008, 0x3242ffff, 0x10560003, 0x31020004, 
-0x10400006, 0x24020305, 0x31020080, 0x54400001, 
-0x34e70010, 0x34e70020, 0x24020905, 0xa4c2000c, 
-0x8ee30000, 0x8ee20004, 0x14620007, 0x3c02b49a, 
-0x8ee20860, 0x54400001, 0x34e70400, 0x3c024b65, 
-0xa000592, 0x34427654, 0x344289ab, 0xacc2001c, 
-0x30e2ffff, 0xacc20010, 0xe0005ac, 0xc02021, 
-0x3242ffff, 0x56102b, 0x1440ff96, 0x0, 
-0x8e620000, 0x8e63fffc, 0x43102a, 0x1440ff3e, 
-0x0, 0x8fbf0044, 0x8fbe0040, 0x8fb7003c, 
-0x8fb60038, 0x8fb50034, 0x8fb40030, 0x8fb3002c, 
-0x8fb20028, 0x8fb10024, 0x8fb00020, 0x3e00008, 
-0x27bd0048, 0x27bdffe8, 0xafbf0014, 0xafb00010, 
-0x8f624450, 0x8f634410, 0xa0005bb, 0x808021, 
-0x8f626820, 0x30422000, 0x10400003, 0x0, 
-0xe0001e3, 0x2021, 0x8f624450, 0x8f634410, 
-0x3042ffff, 0x43102b, 0x1440fff5, 0x0, 
-0x8f630c14, 0x3063000f, 0x2c620002, 0x1440000b, 
-0x0, 0x8f630c14, 0x3c020800, 0x8c421b50, 
-0x3063000f, 0x24420001, 0x3c010800, 0xac221b50, 
-0x2c620002, 0x1040fff7, 0x0, 0xaf705c18, 
-0x8f625c10, 0x30420002, 0x10400009, 0x0, 
-0x8f626820, 0x30422000, 0x1040fff8, 0x0, 
-0xe0001e3, 0x2021, 0xa0005ce, 0x0, 
-0x8fbf0014, 0x8fb00010, 0x3e00008, 0x27bd0018, 
+0xc23021, 0xc02821, 0x61027, 0xa522000a, 
+0x3021, 0x2527000c, 0x4021, 0x94e20000, 
+0x25080001, 0xc23021, 0x2d020004, 0x1440fffb, 
+0x24e70002, 0x95220002, 0x4021, 0x91230009, 
+0x442023, 0x1803821, 0x3082ffff, 0xa4e00010, 
+0x3c040800, 0x948423e4, 0x621821, 0xc33021, 
+0x61c02, 0x30c2ffff, 0x623021, 0x61c02, 
+0x3c020800, 0x944223e0, 0xc34821, 0x441023, 
+0x21fc2, 0x431021, 0x21043, 0x18400010, 
+0x3021, 0x402021, 0x94e20000, 0x24e70002, 
+0xc23021, 0x30e2007f, 0x14400006, 0x25080001, 
+0x8d630000, 0x3c02007f, 0x3442ff80, 0x625824, 
+0x25670008, 0x104102a, 0x1440fff3, 0x0, 
+0x3c020800, 0x944223fc, 0xc23021, 0x3122ffff, 
+0xc23021, 0x61c02, 0x30c2ffff, 0x623021, 
+0x61402, 0xc23021, 0xc04021, 0x61027, 
+0xa5820010, 0xadc00014, 0xa000499, 0xadc00000, 
+0x8dc70010, 0xe04021, 0x11400007, 0x72c02, 
+0xaa3021, 0x61402, 0x30c3ffff, 0x433021, 
+0x61402, 0xc22821, 0x51027, 0xa522000a, 
+0x3c030800, 0x946323e4, 0x3102ffff, 0x1e21021, 
+0x433023, 0xcd3021, 0x61c02, 0x30c2ffff, 
+0x623021, 0x61402, 0xc23021, 0xc04021, 
+0x61027, 0xa5820010, 0x3102ffff, 0x51c00, 
+0x431025, 0xadc20010, 0x3c020800, 0x8c422404, 
+0x10400002, 0x25e2fff2, 0xa5c20034, 0x3c020800, 
+0x8c4223f8, 0x3c040800, 0x8c8423f4, 0x24420001, 
+0x3c010800, 0xac2223f8, 0x3c020800, 0x8c421bd0, 
+0x3303ffff, 0x832021, 0x3c010800, 0xac2423f4, 
+0x431821, 0x62102b, 0x10400003, 0x2482ffff, 
+0x3c010800, 0xac2223f4, 0x3c010800, 0xac231bd0, 
+0x3e00008, 0x27bd0020, 0x27bdffb8, 0x3c050800, 
+0x24a51ba8, 0xafbf0044, 0xafbe0040, 0xafb7003c, 
+0xafb60038, 0xafb50034, 0xafb40030, 0xafb3002c, 
+0xafb20028, 0xafb10024, 0xafb00020, 0x90a30000, 
+0x24020003, 0x146200d5, 0x0, 0x3c090800, 
+0x95291ba6, 0x3c020800, 0x944223e0, 0x3c030800, 
+0x8c631bc0, 0x3c040800, 0x8c841bbc, 0x1221023, 
+0x64182a, 0xa7a9001e, 0x106000c8, 0xa7a20016, 
+0x24be0020, 0x97b6001e, 0x24b30018, 0x24b70014, 
+0x8fc20000, 0x14400008, 0x0, 0x8fc2fff8, 
+0x97a30016, 0x8fc4fff4, 0x431021, 0x82202a, 
+0x148000ba, 0x0, 0x97d50818, 0x32a2ffff, 
+0x104000ad, 0x9021, 0x40a021, 0x8821, 
+0xe00062d, 0x0, 0x403021, 0x14c00007, 
+0x0, 0x3c020800, 0x8c4223ec, 0x24420001, 
+0x3c010800, 0xa00059e, 0xac2223ec, 0x3c100800, 
+0x2118021, 0x8e101bd8, 0x9608000a, 0x31020040, 
+0x10400004, 0x2407180c, 0x8e02000c, 0x2407188c, 
+0xacc20018, 0x31021000, 0x10400004, 0x34e32000, 
+0x81040, 0x3042c000, 0x623825, 0x31020080, 
+0x54400001, 0x34e70010, 0x3c020800, 0x511021, 
+0x8c421be0, 0x3c030800, 0x711821, 0x8c631be4, 
+0x21500, 0x31c00, 0x431025, 0xacc20014, 
+0x96040008, 0x3242ffff, 0x821021, 0x282102a, 
+0x14400002, 0x2b22823, 0x802821, 0x8e020000, 
+0x2459021, 0xacc20000, 0x8e020004, 0xc02021, 
+0x26310010, 0xac820004, 0x30e2ffff, 0xac800008, 
+0xa485000e, 0xac820010, 0x24020305, 0xe0005aa, 
+0xa482000c, 0x3242ffff, 0x54102b, 0x1440ffc0, 
+0x3242ffff, 0xa000596, 0x0, 0x8e620000, 
+0x8e63fffc, 0x43102a, 0x1040006c, 0x0, 
+0x8e62fff0, 0x28900, 0x3c100800, 0x2118021, 
+0xe00062d, 0x8e101bd8, 0x403021, 0x14c00005, 
+0x0, 0x8e62082c, 0x24420001, 0xa00059e, 
+0xae62082c, 0x9608000a, 0x31020040, 0x10400004, 
+0x2407180c, 0x8e02000c, 0x2407188c, 0xacc20018, 
+0x31021000, 0x10400004, 0x34e32000, 0x81040, 
+0x3042c000, 0x623825, 0x3c020800, 0x511021, 
+0x8c421be0, 0x3c030800, 0x711821, 0x8c631be4, 
+0x21500, 0x31c00, 0x431025, 0xacc20014, 
+0x8e63fff4, 0x96020008, 0x432023, 0x3242ffff, 
+0x3083ffff, 0x431021, 0x2c2102a, 0x10400003, 
+0x802821, 0x97a9001e, 0x1322823, 0x8e620000, 
+0x30a4ffff, 0x441021, 0xae620000, 0xa4c5000e, 
+0x8e020000, 0xacc20000, 0x8e020004, 0x8e63fff4, 
+0x431021, 0xacc20004, 0x8e63fff4, 0x96020008, 
+0x641821, 0x62102a, 0x14400006, 0x2459021, 
+0x8e62fff0, 0xae60fff4, 0x24420001, 0xa000579, 
+0xae62fff0, 0xae63fff4, 0xacc00008, 0x3242ffff, 
+0x10560003, 0x31020004, 0x10400006, 0x24020305, 
+0x31020080, 0x54400001, 0x34e70010, 0x34e70020, 
+0x24020905, 0xa4c2000c, 0x8ee30000, 0x8ee20004, 
+0x14620007, 0x3c02b49a, 0x8ee20860, 0x54400001, 
+0x34e70400, 0x3c024b65, 0xa000590, 0x34427654, 
+0x344289ab, 0xacc2001c, 0x30e2ffff, 0xacc20010, 
+0xe0005aa, 0xc02021, 0x3242ffff, 0x56102b, 
+0x1440ff96, 0x0, 0x8e620000, 0x8e63fffc, 
+0x43102a, 0x1440ff3e, 0x0, 0x8fbf0044, 
+0x8fbe0040, 0x8fb7003c, 0x8fb60038, 0x8fb50034, 
+0x8fb40030, 0x8fb3002c, 0x8fb20028, 0x8fb10024, 
+0x8fb00020, 0x3e00008, 0x27bd0048, 0x27bdffe8, 
+0xafbf0014, 0xafb00010, 0x8f624450, 0x8f634410, 
+0xa0005b9, 0x808021, 0x8f626820, 0x30422000, 
+0x10400003, 0x0, 0xe0001e3, 0x2021, 
+0x8f624450, 0x8f634410, 0x3042ffff, 0x43102b, 
+0x1440fff5, 0x0, 0x8f630c14, 0x3063000f, 
+0x2c620002, 0x1440000b, 0x0, 0x8f630c14, 
+0x3c020800, 0x8c421b50, 0x3063000f, 0x24420001, 
+0x3c010800, 0xac221b50, 0x2c620002, 0x1040fff7, 
+0x0, 0xaf705c18, 0x8f625c10, 0x30420002, 
+0x10400009, 0x0, 0x8f626820, 0x30422000, 
+0x1040fff8, 0x0, 0xe0001e3, 0x2021, 
+0xa0005cc, 0x0, 0x8fbf0014, 0x8fb00010, 
+0x3e00008, 0x27bd0018, 0x0, 0x0, 
 0x0, 0x27bdffe8, 0x3c1bc000, 0xafbf0014, 
 0xafb00010, 0xaf60680c, 0x8f626804, 0x34420082, 
 0xaf626804, 0x8f634000, 0x24020b50, 0x3c010800, 
@@ -497,4 +498,3 @@ U32 t3StkOffLdFwData[(0x20/4) + 1] = {
 0x0, 0x0, 0x0, 0x0 };
 
 #endif /* __FW_STKOFFLD_H__  */
-
